def various_items_in_lists(first, second):
    if len(first) != len(second):
        return True
    for i in first:
        if i not in second:
            return True
    return False


class Functions:
    def chr_check(self, a, b, flag=1):
        if type(b) != int:
            b = ord(b)
        # защита от выхода за края + шифр Цезаря
        if type(a) != int:
            a = ord(a)
        a = a + (b * flag)
        if a > 1114111:
            return a % 1114112
        elif a < 0:
            return a % 1114112
        return a

    def crete_key(self, password=0, arr=None):
        normal_arr = ["‌", "‍", "⁡", "⁣", "⁢", "⁤"]
        if arr is None or various_items_in_lists(arr, normal_arr):
            arr = normal_arr
        return_arr = []
        if type(password) == int:
            num = password
        else:
            num = 0
            for i in password:
                num += ord(i)
        for i in range(6, 0, -1):
            return_arr.append(arr[num % i])
            arr.remove(arr[num % i])
            num = num // i
        return return_arr

    def password_check(self, secret, key):
        # увеличение длины ключа до длины доп сообщения
        password = key
        if len(key) > len(secret):
            password = key[:len(secret)]
        elif len(secret) > len(key):
            password = key * (int((len(secret) - len(key)) / len(key)) + 1)
        num = 0
        while len(secret) > len(password):
            password = password + key[num]
            num += 1
        return password

    def crypt_first(self, word):
        # перетасовка текста
        variable_1 = ''
        variable_2 = ''
        for i in range(len(word)):
            if i % 2 == 0:
                variable_1 += word[i]
            else:
                variable_2 += word[i]
        return variable_1 + variable_2

    def decrypt_first(self, word):
        # перетасовка текста обратно
        txt = ''
        if len(word) % 2 == 0:
            for i in range(len(word) // 2):
                txt += word[i]
                txt += word[len(word) // 2 + i]
        else:
            for i in range(len(word) // 2):
                txt += word[i]
                txt += word[len(word) // 2 + i + 1]
            txt += word[len(word) // 2]
        return txt

    def crypt_second(self, word, num=0, flag=1, step=1):
        # num первое значение изменения
        # flag 1 шифровка, -1 расшифровка
        # step шаг шифровки
        # модифицированный шифр Цезаря
        num = num * flag
        txt = ''
        for i in word:
            num += step * flag
            txt += chr(self.chr_check(i, num))
        return txt

    def return_el(self, sms, arr):
        # необходимо для расшифровки невидимых символов
        n, x = 0, 1
        for i in sms:
            n += arr[i] * x
            x = x * 5
        return chr(self.chr_check(n, 0))

    def coding_invisible(self, txt, arr):
        # перевод шифра в невидимые символы
        x = arr[5]
        for i in txt:
            y = ord(i)
            while y != 0:
                z = y % 5
                x += arr[z]
                y = y // 5
            x += arr[5]
        return x

    def decoding_invisible(self, massage, arr):
        # перевод невидимых символов в шифр
        keys = {}
        for i in range(5):
            keys[f'{arr[i]}'] = i
        words = massage.split(arr[5])
        txt = ''
        for i in words:
            txt += self.return_el(i, keys)
        return txt

# functions = Functions()
# print(len(functions.decoding_invisible(functions.coding_invisible('123', functions.crete_key()), functions.crete_key())))



from . import Functions

functions = Functions()


class Crypter:
    def __init__(self, int_seed=0):
        self.seed = int_seed
        self.crypt_keys = functions.crete_key(int_seed)
        self.range = int_seed % 10 + 1

    def change_seed(self, new_int_seed):
        self.seed = new_int_seed
        self.crypt_keys = functions.crete_key(new_int_seed)
        self.range = new_int_seed % 10 + 1

    def create_message(self, sms='', password='1', secret='нет зашифрованного текста'):
        if not password:
            password = ' '
        crypt_keys = functions.crete_key(password, self.crypt_keys)
        password = functions.password_check(secret, password)
        message = ''
        for i in range(len(secret)):
            message += chr(functions.chr_check(secret[i], password[i]))
        message = functions.crypt_first(message)
        message = functions.crypt_second(message, step=self.range)
        message = functions.coding_invisible(message, crypt_keys)
        if len(sms) >= 2:
            return sms[0:len(sms) // 2] + message + sms[len(sms) // 2::]
        return sms + message



from .functions import Functions
from .Crypter import Crypter
from .Decrypter import Decrypter

author_vk = 'https://vk.com/lexinus10001'

__author__ = 'Lexinus'
__version__ = '0.1.3'
__email__ = 'starshenkov3@gmail.com'



from . import Functions

functions = Functions()


class Decrypter:
    def __init__(self, int_seed=0):
        self.seed = int_seed
        self.crypt_keys = functions.crete_key(int_seed)
        self.range = int_seed % 10 + 1

    def change_seed(self, new_int_seed):
        self.seed = new_int_seed
        self.crypt_keys = functions.crete_key(new_int_seed)
        self.range = new_int_seed % 10 + 1

    def create_message(self, txt='', password='1'):
        if not password:
            password = ' '
        crypt_keys = functions.crete_key(password, self.crypt_keys)
        elem = crypt_keys[5]
        txt = txt[txt.index(elem) + 1: txt.rindex(elem)]
        txt = functions.decoding_invisible(txt, crypt_keys)
        txt = functions.crypt_second(txt, flag=-1, step=self.range)
        txt = functions.decrypt_first(txt)
        message = ''
        password = functions.password_check(txt, password)
        for i in range(len(txt)):
            message += chr(functions.chr_check(txt[i], password[i], -1))
        return message



import setuptools

"""
:authors: Lexinus
:license: Apache License, version 2.0, see LICENSE file
:copyright: (c) 2021 Lexinus
"""

version = '0.1.3'

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="steganography_in_text",
    version=version,
    author="Lexinus",
    license='Apache License, version 2.0, see LICENSE file',
    author_email="starshenkov3@gmail.com",
    description="Это модуль, умеющий скрывать текст в тексте",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/red-lexinus/steganography_in_text",
    project_urls={
        "Bug Tracker": "https://github.com/red-lexinus/steganography_in_text",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    package_dir={},
    packages=['steganography_in_text'],
    python_requires=">=3.6",
)



from .Decrypter import Decrypter
from .Crypter import Crypter
from .functions import Functions

functions = Functions()


def test_0():
    assert functions.chr_check(' ', ' ') == 64


def test_1():
    assert functions.chr_check(' ', 12) == 44


def test_2():
    assert functions.password_check('111', '12') == '121'


def test_3():
    assert functions.crypt_first('1234') == '1324'


def test_4():
    assert functions.decrypt_first(functions.crypt_first('12345')) == '12345'


def test_5():
    crypter = Crypter()
    decrypter = Decrypter()
    assert crypter.crypt_keys == decrypter.crypt_keys


def test_6():
    crypter = Crypter()
    decrypter = Decrypter()
    assert decrypter.create_message(crypter.create_message('привет', 'пароль', 'секрет'), 'пароль') == 'секрет'


for i in range(7):
    eval(f'test_{i}()')
